@@ -16,7 +16,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: silcske.c,v 1.47 2002/03/17 14:00:30 priikone Exp $ */
+/* $Id: silcske.c,v 1.47.2.1 2002/04/03 10:36:27 priikone Exp $ */
 
 #include "silcincludes.h"
 #include "silcske.h"
@@ -1100,6 +1100,9 @@ SilcSKEStatus silc_ske_abort(SilcSKE ske, SilcSKEStatus status)
 
   SILC_LOG_DEBUG(("Start"));
 
+  if (status > SILC_SKE_STATUS_INVALID_COOKIE)
+    status = SILC_SKE_STATUS_BAD_PAYLOAD;
+
   packet = silc_buffer_alloc(4);
   silc_buffer_pull_tail(packet, SILC_BUFFER_END(packet));
   silc_buffer_format(packet,
