@@ -2,9 +2,9 @@
 
   packet_receive.c
 
-  Author: Pekka Riikonen <priikone@poseidon.pspt.fi>
+  Author: Pekka Riikonen <priikone@silcnet.org>
 
-  Copyright (C) 1997 - 2001 Pekka Riikonen
+  Copyright (C) 1997 - 2002 Pekka Riikonen
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 /*
  * Server packet routines to handle received packets.
  */
-/* $Id: packet_receive.c,v 1.135.2.1 2002/03/28 20:56:41 priikone Exp $ */
+/* $Id: packet_receive.c,v 1.135.2.3 2002/04/01 22:43:53 johnny Exp $ */
 
 #include "serverincludes.h"
 #include "server_internal.h"
