@@ -2,9 +2,9 @@
 
   server.h
 
-  Author: Pekka Riikonen <priikone@poseidon.pspt.fi>
+  Author: Pekka Riikonen <priikone@silcnet.org>
 
-  Copyright (C) 1997 - 2001 Pekka Riikonen
+  Copyright (C) 1997 - 2002 Pekka Riikonen
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -114,8 +114,6 @@ int silc_server_alloc(SilcServer *new_server);
 void silc_server_free(SilcServer server);
 bool silc_server_init(SilcServer server);
 bool silc_server_rehash(SilcServer server);
-void silc_server_drop(SilcServer server);
-void silc_server_daemonise(SilcServer server);
 void silc_server_run(SilcServer server);
 void silc_server_stop(SilcServer server);
 void silc_server_start_key_exchange(SilcServer server,
