@@ -4,7 +4,7 @@
 
   Author: Pekka Riikonen <priikone@silcnet.org>
 
-  Copyright (C) 1997 - 2001 Pekka Riikonen
+  Copyright (C) 1997 - 2002 Pekka Riikonen
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: server_util.c,v 1.26 2002/03/24 14:18:43 priikone Exp $ */
+/* $Id: server_util.c,v 1.26.2.2 2002/04/01 22:43:53 johnny Exp $ */
 
 #include "serverincludes.h"
 #include "server_internal.h"
@@ -809,6 +809,33 @@ SilcUInt32 silc_server_num_sockets_by_ip(SilcServer server, const char *ip,
   return count;
 }
 
+/* Find number of sockets by IP address indicated by remote host, indicatd
+   by `ip' or `hostname', `port', and `type'.  Returns 0 if socket connections
+   does not exist. If `ip' is provided then `hostname' is ignored. */
+
+SilcUInt32 silc_server_num_sockets_by_remote(SilcServer server, 
+					     const char *ip,
+					     const char *hostname,
+					     SilcUInt16 port,
+					     SilcSocketType type)
+{
+  int i, count;
+
+  if (!ip && !hostname)
+    return 0;
+
+  for (i = 0, count = 0; i < server->config->param.connections_max; i++) {
+    if (server->sockets[i] && 
+	((ip && !strcmp(server->sockets[i]->ip, ip)) ||
+	 (hostname && !strcmp(server->sockets[i]->hostname, hostname))) &&
+	server->sockets[i]->port == port &&
+	server->sockets[i]->type == type)
+      count++;
+  }
+
+  return count;
+}
+
 /* Finds locally cached public key by the public key received in the SKE. 
    If we have it locally cached then we trust it and will use it in the
    authentication protocol.  Returns the locally cached public key or NULL
